@@ -75,6 +75,7 @@ long buf_nbuffers = CM_BUF_BUFFERS;
 long buf_nOrigBuffers;
 long buf_bufferSize = CM_BUF_SIZE;
 long buf_hashSize = CM_BUF_HASHSIZE;
+int buf_cacheType = CM_BUF_CACHETYPE_FILE;
 
 #ifndef DJGPP
 static
@@ -295,6 +296,7 @@ long buf_Init(cm_buf_ops_t *opsp)
 		sectorSize = 1;
 
 #ifndef DJGPP
+        if(buf_cacheType == CM_BUF_CACHETYPE_FILE) {
 		/* Reserve buffer space by mapping cache file */
 		psa = CreateCacheFileSA();
 		hf = CreateFile(cm_CachePath,
@@ -309,6 +311,9 @@ long buf_Init(cm_buf_ops_t *opsp)
 			return CM_ERROR_INVAL;
 		}
 		FreeCacheFileSA(psa);
+        } else { /* buf_cacheType == CM_BUF_CACHETYPE_VIRTUAL */
+            hf = INVALID_HANDLE_VALUE;
+        }
 		CacheHandle = hf;
 		hm = CreateFileMapping(hf,
 			NULL,
@@ -327,7 +332,7 @@ long buf_Init(cm_buf_ops_t *opsp)
 			0, 0,
 			buf_nbuffers * buf_bufferSize);
 		if (data == NULL) {
-			CloseHandle(hf);
+			if(hf != INVALID_HANDLE_VALUE) CloseHandle(hf);
 			CloseHandle(hm);
 			return CM_ERROR_INVAL;
 		}
@@ -404,6 +409,15 @@ long buf_AddBuffers(long nbuffers)
     afsi_log("%d buffers being added to the existing cache of size %d",
               nbuffers, buf_nbuffers);
 
+    if (buf_cacheType == CM_BUF_CACHETYPE_VIRTUAL) {
+        /* The size of a virtual cache cannot be changed after it has
+         * been created.  Subsequent calls to MapViewofFile() with
+         * an existing mapping object name would not allow the 
+         * object to be resized.  Return failure immediately.
+	 */
+        return CM_ERROR_INVAL;
+    }
+
 	/*
 	 * Cache file mapping constrained by
 	 * system allocation granularity;
@@ -477,11 +491,14 @@ long buf_AddBuffers(long nbuffers)
  */
 long buf_SetNBuffers(long nbuffers)
 {
-	if (nbuffers < 10) return CM_ERROR_INVAL;
-        if (nbuffers == buf_nbuffers) return 0;
+    if (nbuffers < 10) 
+        return CM_ERROR_INVAL;
+    if (nbuffers == buf_nbuffers) 
+        return 0;
         else if (nbuffers > buf_nbuffers)
 		return buf_AddBuffers(nbuffers - buf_nbuffers);
-        else return CM_ERROR_INVAL;
+    else 
+        return CM_ERROR_INVAL;
 }
 
 /* release a buffer.  Buffer must be referenced, but unlocked. */
@@ -502,24 +519,27 @@ void buf_WaitIO(cm_buf_t *bp)
 		if (!(bp->flags & (CM_BUF_READING | CM_BUF_WRITING)))
 			break;
 		
-                /* otherwise I/O is happening, but some other thread is waiting for
-                 * the I/O already.  Wait for that guy to figure out what happened,
-                 * and then check again.
-                 */
-                bp->flags |= CM_BUF_WAITING;
-                osi_SleepM((long) bp, &bp->mx);
-                lock_ObtainMutex(&bp->mx);
-		osi_Log1(buf_logp, "buf_WaitIO conflict wait done for 0x%x", bp);
-        }
-        
-        /* if we get here, the IO is done, but we may have to wakeup people waiting for
-         * the I/O to complete.  Do so.
+        /* otherwise I/O is happening, but some other thread is waiting for
+         * the I/O already.  Wait for that guy to figure out what happened,
+         * and then check again.
          */
-        if (bp->flags & CM_BUF_WAITING) {
+        if ( bp->flags & CM_BUF_WAITING ) 
+            osi_Log1(buf_logp, "buf_WaitIO CM_BUF_WAITING already set for 0x%x", bp);
+
+        bp->flags |= CM_BUF_WAITING;
+        osi_SleepM((long) bp, &bp->mx);
+        lock_ObtainMutex(&bp->mx);
+		osi_Log1(buf_logp, "buf_WaitIO conflict wait done for 0x%x", bp);
+    }
+        
+    /* if we get here, the IO is done, but we may have to wakeup people waiting for
+     * the I/O to complete.  Do so.
+     */
+    if (bp->flags & CM_BUF_WAITING) {
 		bp->flags &= ~CM_BUF_WAITING;
-                osi_Wakeup((long) bp);
-        }
-        osi_Log1(buf_logp, "WaitIO finished wait for bp 0x%x", (long) bp);
+        osi_Wakeup((long) bp);
+    }
+    osi_Log1(buf_logp, "WaitIO finished wait for bp 0x%x", (long) bp);
 }
 
 /* code to drop reference count while holding buf_globalLock */
@@ -635,8 +655,7 @@ void buf_Recycle(cm_buf_t *bp)
 	 * have any lock conflicts, so we can grab the buffer lock out of
 	 * order in the locking hierarchy.
 	 */
-	osi_Log2(buf_logp,
-		"buf_Recycle recycles 0x%x, off 0x%x",
+    osi_Log2( buf_logp, "buf_Recycle recycles 0x%x, off 0x%x",
 		bp, bp->offset.LowPart);
 
 	osi_assert(bp->refCount == 0);
@@ -1279,7 +1298,6 @@ long buf_Truncate(cm_scache_t *scp, cm_user_t *userp, cm_req_t *reqp,
 			}
 
                         lock_ReleaseWrite(&buf_globalLock);
-
                 }
 		
                 lock_ReleaseMutex(&scp->mx);
@@ -1371,44 +1389,45 @@ long buf_CleanVnode(struct cm_scache *scp, cm_user_t *userp, cm_req_t *reqp)
 {
 	long code;
 	cm_buf_t *bp;		/* buffer we're hacking on */
-        cm_buf_t *nbp;		/* next one */
+    cm_buf_t *nbp;		/* next one */
 	long i;
 
 	i = BUF_FILEHASH(&scp->fid);
 
 	code = 0;
 	lock_ObtainWrite(&buf_globalLock);
-        bp = buf_fileHashTablepp[i];
-        if (bp) bp->refCount++;
-        lock_ReleaseWrite(&buf_globalLock);
+    bp = buf_fileHashTablepp[i];
+    if (bp) bp->refCount++;
+    lock_ReleaseWrite(&buf_globalLock);
 	for(; bp; bp = nbp) {
 		/* clean buffer synchronously */
 		if (cm_FidCmp(&bp->fid, &scp->fid) == 0) {
 			if (userp) {
+                cm_HoldUser(userp);
 				lock_ObtainMutex(&bp->mx);
-				if (bp->userp) cm_ReleaseUser(bp->userp);
-                                bp->userp = userp;
+				if (bp->userp) 
+                    cm_ReleaseUser(bp->userp);
+                bp->userp = userp;
 				lock_ReleaseMutex(&bp->mx);
-                                cm_HoldUser(userp);
-                        }
+            }
 			buf_CleanAsync(bp, reqp);
-	                buf_CleanWait(bp);
-                        lock_ObtainMutex(&bp->mx);
+            buf_CleanWait(bp);
+            lock_ObtainMutex(&bp->mx);
 			if (bp->flags & CM_BUF_ERROR) {
 				if (code == 0 || code == -1) code = bp->error;
-                                if (code == 0) code = -1;
-                        }
-                        lock_ReleaseMutex(&bp->mx);
+                if (code == 0) code = -1;
+            }
+            lock_ReleaseMutex(&bp->mx);
 		}
 
 		lock_ObtainWrite(&buf_globalLock);
 		buf_LockedRelease(bp);
-                nbp = bp->fileHashp;
-                if (nbp) nbp->refCount++;
+        nbp = bp->fileHashp;
+        if (nbp) nbp->refCount++;
 		lock_ReleaseWrite(&buf_globalLock);
 	}	/* for loop over a bunch of buffers */
 	
-        /* done */
+    /* done */
 	return code;
 }
 
